#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "IOTCClient.h"
#include <windows.h>

#define SEARCH_DEVICE_TIME 6000
#define SEARCH_DEVICE_TIME_ONE_ROUND 100

static int gSID = -1;

void WINAPI thread_ForSessionRead(void *arg)
{
	int ChID = *(int *)arg;
	int SID = gSID;
	char buf[IOTC_MAX_PACKET_SIZE];
	int ret;
	printf("[thread_ForSessionRead] started channel[%d]\n", ChID);

	while(1)
	{
		ret = IOTC_Session_Read(SID, buf, IOTC_MAX_PACKET_SIZE, 6000, ChID);

		if(ret > -1)
		{
			buf[ret] = 0;
			if(strcmp(buf, "end") == 0 || strcmp(buf, "END") == 0 )
				break;
			printf("[IOTC_Session_Read] SID[%d] ChID[%d] Size[%d] Data=> %s\n", SID, ChID, ret, buf);
		}
		// check error code
		else if(ret == IOTC_ER_SESSION_CLOSE_BY_REMOTE)
		{
			printf("[thread_ForSessionRead] remote site close this session, SID[%d]\n", SID);
			break;
		}
		else if(ret == IOTC_ER_REMOTE_TIMEOUT_DISCONNECT)
		{
			printf("[thread_ForSessionRead] disconnected due to remote site no response for a while SID[%d]\n", SID);
			break;
		}
		else if(ret == IOTC_ER_INVALID_SID)
		{
			printf("Session already closed by other thread\n");
			break;
		}
		else if(ret == IOTC_ER_CH_NOT_ON)
			IOTC_Session_Channel_ON(SID, ChID);
	}

	printf("[thread_ForSessionRead] exit channel[%d]\n", ChID);

	return;
}

void WINAPI thread_ForSessionWrite(void *arg)
{
	int ChID = *(int *)arg;
	int SID = gSID;
	char buf[IOTC_MAX_PACKET_SIZE];
	int ret;
	int i;

	printf("[thread_ForSessionWrite] started channel[%d]\n", ChID);

	// wait a minute to send for avoid packet lost
	Sleep(3000);

	for(i = 0; i < 3; i++)
	{
		sprintf(buf, "Hello World CH[%d] NO[%d]", ChID, i);
		ret = IOTC_Session_Write(SID, buf, strlen(buf), ChID);
		if(ret == IOTC_ER_CH_NOT_ON)
		{
			printf("IOTC_ER_CH_NOT_ON!!!!!!!!!!1\n");
			IOTC_Session_Channel_ON(SID, ChID);
		}
		else if(ret < 0) // other error < 0 means this session cant use anymore
		{
			printf("IOTC_Session_Write error, code[%d]\n", ret);
			return;
		}
	}

	sprintf(buf, "end");
	IOTC_Session_Write(SID, buf, strlen(buf), ChID);

	printf("[thread_ForSessionWrite] exit channel[%d]\n", ChID);

	return;
}

int main(int argc, char *argv[])
{
	int ret;
	IOTCConnectInput auth_option = {0};
	int use_key = 0;
	char *license_key = (char *)argv[1];
	char *UID = (char *)argv[2];

	if (argc < 3)
	{
		printf("IOTCAPIs_Client <LICENSE_KEY> <UID> [AUTH_KEY]\n");
		return 0;
	}

	auth_option.cb = sizeof(IOTCConnectInput);
	auth_option.authentication_type = AUTHENTICATE_BY_KEY;
	memset(auth_option.auth_key, '0', 8);
	auth_option.timeout = 0;

	if (argc >3)
	{

		if (strlen(argv[3]) == 8)
		{
			use_key = 1;
			
			strncpy(auth_option.auth_key, argv[3], 8);
			printf("Connect with Key=%s\n", argv[3]);
		}
		else
		{
			printf("Invalid Key=%s, Connect without Key\n", argv[3]);
		}

	}
	else
	{
		printf("Connect without Key\n");
	}

    ret = TUTK_SDK_Set_License_Key(license_key);
    if (ret != TUTK_ER_NoERROR)
    {
        printf("TUTK_SDK_Set_License_Key()=%d ,exit...!!\n", ret);
        return 0;
    }

	// Must be first call before IOTC_Initialize2(), but not a must
	IOTC_Set_Max_Session_Number(32);

	// use which Master base on location, port 0 means to get a random port
	ret = IOTC_Initialize2(0);
	printf("Step 1: call IOTC_Initialize2(), ret = %d\n", ret);
	if(ret != IOTC_ER_NoERROR)
	{
		printf("IOTCAPIs_Client exit...!!\n");
		return 0;
	}

	gSID = IOTC_Get_SessionID();
	if(gSID < 0)
	{
		printf("IOTC_Get_SessionID error code [%d]\n", gSID);
		return 0;
	}

	if(use_key)
		ret = IOTC_Connect_ByUIDEx(UID, gSID, &auth_option);
	else
		ret = IOTC_Connect_ByUID_Parallel(UID, gSID);

	printf("Step 2: call IOTC_Connect_ByUIDEx(%s).......\n", UID);

	struct st_SInfoEx Sinfo;
    Sinfo.size = sizeof(struct st_SInfoEx);
	IOTC_Session_Check_Ex(gSID, &Sinfo);
	if(Sinfo.Mode ==0)
		printf("Device info %s:%d[%s] Mode=P2P\n",Sinfo.RemoteIP, Sinfo.RemotePort, Sinfo.UID);
	else if(Sinfo.Mode == 1)
		printf("Device info %s:%d[%s] Mode=RLY\n",Sinfo.RemoteIP, Sinfo.RemotePort, Sinfo.UID);
	else if(Sinfo.Mode == 2)
		printf("Device info %s:%d[%s] Mode=LAN\n",Sinfo.RemoteIP, Sinfo.RemotePort, Sinfo.UID);

	if(ret >= 0)
	{
		printf("IOTC Connect OK SID[%d]\n", gSID);
		int ChID1 = 0, ChID2 = 1;
		// channel 0 always on, so just turn on channel 1
		IOTC_Session_Channel_ON(gSID, 1);

		HANDLE ThreadRead_ID1, ThreadRead_ID2;
		HANDLE ThreadWrt_ID1, ThreadWrt_ID2;
		ThreadRead_ID1 = CreateThread(NULL, 0, thread_ForSessionRead, (void*)&ChID1, 0, NULL);
		ThreadRead_ID2 = CreateThread(NULL, 0, thread_ForSessionRead, (void*)&ChID2, 0, NULL);
		ThreadWrt_ID1 = CreateThread(NULL, 0, thread_ForSessionWrite, (void*)&ChID1, 0, NULL);
		ThreadWrt_ID2 = CreateThread(NULL, 0, thread_ForSessionWrite, (void*)&ChID2, 0, NULL);
		// wait all thread exit for confirm all data already read/write
		WaitForSingleObject(ThreadWrt_ID1, INFINITE);
		WaitForSingleObject(ThreadWrt_ID2, INFINITE);
		WaitForSingleObject(ThreadRead_ID1, INFINITE);
		WaitForSingleObject(ThreadRead_ID2, INFINITE);
		CloseHandle(ThreadWrt_ID1);
		CloseHandle(ThreadWrt_ID2);
		CloseHandle(ThreadRead_ID1);
		CloseHandle(ThreadRead_ID2);
		IOTC_Session_Close(gSID);
	}
	else
		printf("IOTC_Connect error code[%d]\n", ret);

	IOTC_DeInitialize();

	return 0;
}
